-- -----------------------------------------------------
-- Database QuizFaber x Oracle , db version : 2
-- -----------------------------------------------------
-- Project : QuizFaber
-- Licence : MIT
-- Author  : Luca Galli
-- Email   : info@quizfaber.com
-- -----------------------------------------------------

-- -----------------------------------------------------
-- Table `QuizState`
-- -----------------------------------------------------
CREATE TABLE QuizState (
  Code INT NOT NULL,
  Description VARCHAR(30) NULL,
  PRIMARY KEY (Code));

INSERT INTO QuizState (Code,Description) SELECT * FROM (SELECT 0,'Draft' FROM DUAL) WHERE NOT EXISTS (SELECT * FROM QuizState WHERE Code=0);
INSERT INTO QuizState (Code,Description) SELECT * FROM (SELECT 1,'Completed' FROM DUAL)  WHERE NOT EXISTS (SELECT * FROM QuizState WHERE Code=1);
INSERT INTO QuizState (Code,Description) SELECT * FROM (SELECT 2,'Candidate' FROM DUAL)WHERE NOT EXISTS (SELECT * FROM QuizState WHERE Code=2);
INSERT INTO QuizState (Code,Description) SELECT * FROM (SELECT 10,'Published' FROM DUAL)  WHERE NOT EXISTS (SELECT * FROM QuizState WHERE Code=10);
INSERT INTO QuizState (Code,Description) SELECT * FROM (SELECT 98,'Archived' FROM DUAL) WHERE NOT EXISTS (SELECT * FROM QuizState WHERE Code=98);
INSERT INTO QuizState (Code,Description) SELECT * FROM (SELECT 99,'Deleted' FROM DUAL)  WHERE NOT EXISTS (SELECT * FROM QuizState WHERE Code=99);

-- -----------------------------------------------------
-- Table Quizzes
-- -----------------------------------------------------
CREATE TABLE Quizzes (
  QZ_ID INT NOT NULL,
  quizname VARCHAR(255) NOT NULL,
  Title VARCHAR(1024) NOT NULL,
  IntroText CLOB NULL,
  EpiText CLOB NULL,
  NumQuestions INT NOT NULL,
  Author VARCHAR(255) NULL,
  Argument VARCHAR(255) NULL,
  Season VARCHAR(255) NULL,
  "STATE" INT DEFAULT 0 NOT NULL,
  Link VARCHAR(255) NULL,
  Properties CLOB NULL,
  DateCreated TIMESTAMP NULL,
  DateModified TIMESTAMP NULL,
  PRIMARY KEY (QZ_ID),
  CONSTRAINT FK_QuizState
    FOREIGN KEY (State)
    REFERENCES QuizState (Code));

 CREATE SEQUENCE QUIZ_ID_SEQ;
  
 CREATE OR REPLACE TRIGGER QUIZ_ID_TRIG 
	BEFORE INSERT ON Quizzes 
	FOR EACH ROW
	WHEN (new.QZ_ID IS NULL)
	BEGIN
	  SELECT QUIZ_ID_SEQ.NEXTVAL
	  INTO   :new.QZ_ID
	  FROM   dual;
	END;
/


-- -----------------------------------------------------
-- Table PersonRole
-- -----------------------------------------------------
CREATE TABLE PersonRole (
  ID INT NOT NULL,
  RoleName VARCHAR(80) NOT NULL,
  RoleDescription VARCHAR(500) NULL,
  IsEnabled CHAR(1) NOT NULL,
  PRIMARY KEY (ID));


-- -----------------------------------------------------
-- Table Person
-- -----------------------------------------------------

CREATE TABLE Person (
  ID INT NOT NULL,
  PersonName VARCHAR(255) NOT NULL,
  UserIdentity VARCHAR(255) NOT NULL,
  UserPassword VARCHAR(255) NOT NULL,
  UserRole INT NOT NULL,
  Info CLOB NULL,
  IsEnabled CHAR(1) NOT NULL,
  Source VARCHAR(1) NULL,
  DateCreated TIMESTAMP NULL,
  DateModified TIMESTAMP NULL,
  DateLastAccess TIMESTAMP NULL,
  PRIMARY KEY (ID),
  UNIQUE KEY indexUserIdentity (UserIdentity),
  CONSTRAINT UserRole_FK
    FOREIGN KEY(UserRole) 
	REFERENCES PersonRole(ID));

CREATE SEQUENCE USER_ID_SEQ;
  
 CREATE OR REPLACE TRIGGER USER_ID_TRIG 
	BEFORE INSERT ON Person 
	FOR EACH ROW
	WHEN (new.ID IS NULL)
	BEGIN
	  SELECT USER_ID_SEQ.NEXTVAL
	  INTO   :new.ID
	  FROM   dual;
	END;
/

-- -----------------------------------------------------
-- Table Categories
-- -----------------------------------------------------
CREATE TABLE Categories (
  QZ_ID INT NOT NULL,
  CategoryName VARCHAR(80) NOT NULL,
  Description CLOB NULL,
  MaxQuestForQuiz INT NULL,
  IsRandom NUMBER(1,0) NULL,
  PRIMARY KEY (CategoryName, QZ_ID),
  CONSTRAINT FK_CategoryQuiz
    FOREIGN KEY (QZ_ID)
    REFERENCES Quizzes (QZ_ID)
    ON DELETE CASCADE);



-- -----------------------------------------------------
-- Table Questions
-- -----------------------------------------------------
CREATE TABLE Questions (
  QZ_ID INT NOT NULL,
  Num INT NOT NULL,
  CategoryName VARCHAR(80) NOT NULL,
  TextQuestion CLOB NOT NULL,
  QuestionName VARCHAR(256) NULL,
  Type INT NOT NULL,
  Weight FLOAT NOT NULL,
  Include NUMBER(1,0) NOT NULL,
  TextFormat INT NOT NULL,
  SortAnswers NUMBER(1,0) NOT NULL,
  TextAbove CLOB NULL,
  TextBelow CLOB NULL,
  Remark CLOB NULL,
  PutSeparator NUMBER(1,0) NULL,
  Timeout INT NULL,
  Html CLOB NULL,
  PRIMARY KEY (QZ_ID, Num),
  --INDEX Category_FK_idx (CategoryName ASC, QZ_ID ASC),
  CONSTRAINT Quest_FK
    FOREIGN KEY (QZ_ID)
    REFERENCES Quizzes (QZ_ID)
    ON DELETE CASCADE, 
  CONSTRAINT Category_FK
    FOREIGN KEY (CategoryName , QZ_ID)
    REFERENCES Categories (CategoryName , QZ_ID)
    ON DELETE CASCADE
    );



-- -----------------------------------------------------
-- Table ChoiceAnswers
-- -----------------------------------------------------
CREATE TABLE ChoiceAnswers (
  QZ_ID INT NOT NULL,
  NumQst INT NOT NULL,
  TypeOfAns INT NOT NULL,
  SingleAns NUMBER(1,0) NOT NULL,
  PRIMARY KEY (QZ_ID, NumQst),
  CONSTRAINT ChoiceAns_FK
    FOREIGN KEY (QZ_ID , NumQst)
    REFERENCES Questions (QZ_ID , Num)
    ON DELETE CASCADE
    );



-- -----------------------------------------------------
-- Table ChoiceAnswerItems
-- -----------------------------------------------------
CREATE TABLE ChoiceAnswerItems (
  QZ_ID INT NOT NULL,
  NumQst INT NOT NULL,
  NumAns INT NOT NULL,
  TextFormat INT NOT NULL,
  Ans CLOB NOT NULL,
  RightAns NUMBER(1,0) NOT NULL,
  Hint CLOB NULL,
  Rem CLOB NULL,
  PointsIn INT NULL,
  PointsOut INT NULL,
  Html CLOB NULL,
  PRIMARY KEY (QZ_ID, NumQst, NumAns),
  CONSTRAINT ChoiceItems_FK
    FOREIGN KEY (QZ_ID , NumQst)
    REFERENCES ChoiceAnswers (QZ_ID , NumQst)
    ON DELETE CASCADE
    );



-- -----------------------------------------------------
-- Table BooleanAnswerItems
-- -----------------------------------------------------
CREATE TABLE BooleanAnswerItems (
  QZ_ID INT NOT NULL,
  NumQst INT NOT NULL,
  NumAns INT NOT NULL,
  Sentence CLOB NOT NULL,
  TrueAns NUMBER(1,0) NOT NULL,
  TextFormat INT NOT NULL,
  Html CLOB NULL,
  PRIMARY KEY (QZ_ID, NumQst, NumAns),
  CONSTRAINT BoolAns_FK
    FOREIGN KEY (QZ_ID , NumQst)
    REFERENCES Questions (QZ_ID , Num)
    ON DELETE CASCADE
    );



-- -----------------------------------------------------
-- Table FillGapAnswers
-- -----------------------------------------------------
CREATE TABLE FillGapAnswers (
  QZ_ID INT NOT NULL,
  NumQst INT NOT NULL,
  Phrase CLOB NOT NULL,
  Type INT NOT NULL,
  TextFormat INT NOT NULL,
  Html CLOB NULL,
  PRIMARY KEY (QZ_ID, NumQst),
  CONSTRAINT FillGap_FK
    FOREIGN KEY (QZ_ID , NumQst)
    REFERENCES Questions (QZ_ID , Num)
    ON DELETE CASCADE
    );



-- -----------------------------------------------------
-- Table MatchingAnswers
-- -----------------------------------------------------
CREATE TABLE MatchingAnswers (
  QZ_ID INT NOT NULL,
  NumQst INT NOT NULL,
  Type INT NOT NULL,
  PRIMARY KEY (QZ_ID, NumQst),
  CONSTRAINT Match_FK
    FOREIGN KEY (QZ_ID , NumQst)
    REFERENCES Questions (QZ_ID , Num)
    ON DELETE CASCADE
    );



-- -----------------------------------------------------
-- Table PairNames
-- -----------------------------------------------------
CREATE TABLE PairNames (
  QZ_ID INT NOT NULL,
  NumQst INT NOT NULL,
  NumAns INT NOT NULL,
  LeftItem CLOB NOT NULL,
  RightItem CLOB NOT NULL,
  TextFormat INT NOT NULL,
  Html CLOB NULL,
  PRIMARY KEY (QZ_ID, NumQst, NumAns),
  CONSTRAINT PairNames_FK
    FOREIGN KEY (QZ_ID , NumQst)
    REFERENCES MatchingAnswers (QZ_ID , NumQst)
    ON DELETE CASCADE
    );


-- -----------------------------------------------------
-- Table CustomQuestionTypes
-- -----------------------------------------------------

CREATE TABLE CustomQuestionTypes (
  QZ_ID INT NOT NULL,
  CustomName VARCHAR(255) NOT NULL,
  Description CLOB NULL,
  Version INT NULL,
  Subversion INT NULL,
  IconData CLOB NULL,
  IncludeConfirmButton NUMBER(1,0) NULL,
  ValuateRightWrong NUMBER(1,0) NULL,
  MaxScore INT NULL,
  MinScore INT NULL,
  JscriptCode CLOB NULL,
  HtmlCode CLOB NULL,
  PRIMARY KEY (QZ_ID, CustomName),
  CONSTRAINT FK_CustomQuestionType_Quiz
    FOREIGN KEY (QZ_ID)
    REFERENCES Quizzes (QZ_ID)
    ON DELETE CASCADE);



-- -----------------------------------------------------
-- Table CustomParameters
-- -----------------------------------------------------

CREATE TABLE CustomParameters (
  QZ_ID INT NOT NULL,
  CustomName VARCHAR(255) NOT NULL,
  ParamName VARCHAR(255) NOT NULL,
  IsEncrypt NUMBER(1,0) NOT NULL,
  Type INT NOT NULL,
  DefValue VARCHAR(255) NULL,
  PRIMARY KEY (QZ_ID, CustomName, ParamName),
  CONSTRAINT FK_Params
    FOREIGN KEY (QZ_ID , CustomName)
    REFERENCES CustomQuestionTypes (QZ_ID , CustomName)
    ON DELETE CASCADE
 );



-- -----------------------------------------------------
-- Table CustomHeadTags
-- -----------------------------------------------------

CREATE TABLE CustomHeadTags (
  QZ_ID INT NOT NULL,
  CustomName VARCHAR(255) NOT NULL,
  FileName VARCHAR(255) NOT NULL,
  Content CLOB NOT NULL,
  ContentType INT NOT NULL,
  PRIMARY KEY (FileName, CustomName, QZ_ID),
  CONSTRAINT FK_CustomHeadTags
    FOREIGN KEY (QZ_ID , CustomName)
    REFERENCES CustomQuestionTypes (QZ_ID , CustomName)
    ON DELETE CASCADE
);



-- -----------------------------------------------------
-- Table CustomAnswers
-- -----------------------------------------------------

CREATE TABLE CustomAnswers (
  QZ_ID INT NOT NULL,
  NumQst INT NOT NULL,
  CustomName VARCHAR(255) NOT NULL,
  ParamName VARCHAR(255) NOT NULL,
  ParamValue VARCHAR(255) NOT NULL,
  PRIMARY KEY (QZ_ID, NumQst, CustomName, ParamName),
  CONSTRAINT FK_CustomAns_To_Question
    FOREIGN KEY (QZ_ID , NumQst)
    REFERENCES Questions (QZ_ID , Num)
    ON DELETE CASCADE,
  CONSTRAINT FK_CustomAns_To_CustomParams
    FOREIGN KEY (QZ_ID , CustomName , ParamName)
    REFERENCES CustomParameters (QZ_ID , CustomName , ParamName)
    ON DELETE CASCADE
);



-- -----------------------------------------------------
-- Table Slides
-- -----------------------------------------------------
CREATE TABLE Slides (
  QZ_ID INT NOT NULL,
  NumQst INT NOT NULL,
  SlideName VARCHAR(80) NOT NULL,
  Content CLOB NOT NULL,
  TextFormat INT NOT NULL,
  BeforeQst NUMBER(1,0) NOT NULL,
  IsExternal NUMBER(1,0) NOT NULL,
  HtmlFileName VARCHAR(255) NULL,
  SpeechFileName VARCHAR(255) NULL,
  Timeout INT NULL,
  PRIMARY KEY (QZ_ID, NumQst, SlideName),
  CONSTRAINT FK_Slide_To_Question
    FOREIGN KEY (QZ_ID , NumQst)
    REFERENCES Questions (QZ_ID , Num)
    ON DELETE CASCADE
);



-- -----------------------------------------------------
-- Table SlideDocs
-- -----------------------------------------------------
CREATE TABLE SlideDocs (
  QZ_ID INT NOT NULL,
  NumQst INT NOT NULL,
  SlideName VARCHAR(80) NOT NULL,
  DocFileName VARCHAR(255) NOT NULL,
  LinkName VARCHAR(500) NOT NULL,
  PRIMARY KEY (DocFileName, QZ_ID, NumQst, SlideName),
  CONSTRAINT FK_Doc_To_Slide
    FOREIGN KEY (QZ_ID , NumQst , SlideName)
    REFERENCES Slides (QZ_ID , NumQst , SlideName)
    ON DELETE CASCADE
);

-- -----------------------------------------------------
-- Table QuizResult
-- -----------------------------------------------------
CREATE TABLE QuizResult (
  ID INT NOT NULL,
  QuizName VARCHAR(255) NOT NULL,
  QuizTitle VARCHAR(1024) NOT NULL,
  QuizID INT NULL,
  UserName VARCHAR(255) NOT NULL,
  UserLogin VARCHAR(255) NOT NULL,
  DateCompleted TIMESTAMP NOT NULL,
  DateReceived TIMESTAMP NOT NULL,
  QuestsNum INT NOT NULL,
  HighestMark INT NOT NULL,
  LowestMark INT DEFAULT 0 NOT NULL,
  RoundMark FLOAT(12) DEFAULT 1 NOT NULL,
  FinalMark FLOAT(12) NOT NULL,
  ReviewMark FLOAT(12) NULL,
  ReviewDate TIMESTAMP NULL,
  FinalPoints FLOAT(12) NULL,
  ReviewPoints FLOAT(12) NULL,
  TotalTime INT NOT NULL,
  ElapsedTime INT NOT NULL,
  RightQuestsNum INT NOT NULL,
  WrongQuestsNum INT NOT NULL,
  NotAnsweredQuestsNum INT NULL,
  NotValuatedQuestsNum INT NULL,
  NumOfRetake INT NOT NULL,
  IsDuplicated NUMBER(1,0) NULL,
  PRIMARY KEY (ID));


 CREATE SEQUENCE RESULT_ID_SEQ;
  
 CREATE OR REPLACE TRIGGER RESULT_ID_TRIG 
	BEFORE INSERT ON QuizResult 
	FOR EACH ROW
	WHEN (new.ID IS NULL)
	BEGIN
	  SELECT RESULT_ID_SEQ.NEXTVAL
	  INTO   :new.ID
	  FROM   dual;
	END;
/
-- -----------------------------------------------------
-- Table QuizResultReport
-- -----------------------------------------------------
CREATE TABLE QuizResultReport (
  ID INT NOT NULL,
  Origin CLOB NULL,
  Report CLOB NULL,
  Charset VARCHAR(255) NULL,
  Language VARCHAR(10) NULL,
  PRIMARY KEY (ID),
  CONSTRAINT FK_Report_QuizResult
    FOREIGN KEY (ID)
    REFERENCES QuizResult (ID)
    ON DELETE CASCADE);

-- -----------------------------------------------------
-- Table QuizResultQuestion
-- -----------------------------------------------------
CREATE TABLE QuizResultQuestion (
  ID INT NOT NULL,
  IdQuizResult INT NOT NULL,
  QuestNum INT NOT NULL,
  QuestType INT NOT NULL,
  Weight FLOAT NOT NULL,
  ShortTextQuestion VARCHAR(500) NULL,
  Valid INT NOT NULL,
  Score INT NOT NULL,
  MaxScore INT NOT NULL,
  MinScore INT NOT NULL,
  Points FLOAT(12) NULL,
  Feedback CLOB NULL,
  IsCancelled NUMBER(1,0) NULL,
  PRIMARY KEY (ID),
  CONSTRAINT FK_ResultQuestion_To_Result
    FOREIGN KEY (IdQuizResult)
    REFERENCES QuizResult(ID)
    ON DELETE CASCADE);
	
CREATE SEQUENCE RESULT_QUESTION_ID_SEQ;
  
 CREATE OR REPLACE TRIGGER RESULT_QUESTION_ID_TRIG 
	BEFORE INSERT ON QuizResultQuestion 
	FOR EACH ROW
	WHEN (new.ID IS NULL)
	BEGIN
	  SELECT RESULT_QUESTION_ID_SEQ.NEXTVAL
	  INTO   :new.ID
	  FROM   dual;
	END;
/
-- -----------------------------------------------------
-- Table QuizResultAnswer
-- -----------------------------------------------------
CREATE TABLE QuizResultAnswer (
  IdResultQuestion INT NOT NULL,
  AnswerNum INT NOT NULL,
  Choice VARCHAR(500) NOT NULL,
  Valuation VARCHAR(500) NOT NULL,
  IsGuess CHAR(1) NOT NULL,
  Score INT NULL,
  AdditionalText CLOB NULL,
  ShortTextAnswer VARCHAR(1000) NULL,
  ShortTextRemark VARCHAR(1000) NULL,
  PRIMARY KEY (IdResultQuestion,AnswerNum),
  CONSTRAINT FK_ResultAnswer_To_ResultQuestion
    FOREIGN KEY (IdResultQuestion)
    REFERENCES QuizResultQuestion(ID)
    ON DELETE CASCADE);

-- -----------------------------------------------------
-- Table `QuizSession`
-- -----------------------------------------------------
CREATE TABLE QuizSession (
  PersonID INT NOT NULL,
  SessionID INT NOT NULL,
  DateCreated TIMESTAMP NOT NULL,
  IpLogin VARCHAR(46) NULL,
  SessionData CLOB NULL,
  DateLastUpdate TIMESTAMP NULL,
  QuizID INT NULL,
  IpLogout VARCHAR(46) NULL,
  UserAgent VARCHAR(255) NULL,
  IsRecoverable NUMBER(1,0) NULL,
  PRIMARY KEY (SessionID),
  CONSTRAINT FK_QuizSessionPerson
    FOREIGN KEY (PersonID)
    REFERENCES Person (ID),
  CONSTRAINT FK_QuizSessionQuizResult
    FOREIGN KEY (QuizID)
    REFERENCES QuizResult (ID));


 CREATE SEQUENCE SESSION_ID_SEQ;
  
 CREATE OR REPLACE TRIGGER SESSION_ID_TRIG 
	BEFORE INSERT ON QuizSession 
	FOR EACH ROW
	WHEN (new.SessionID IS NULL)
	BEGIN
	  SELECT SESSION_ID_SEQ.NEXTVAL
	  INTO   :new.SessionID
	  FROM   dual;
	END;


-- -----------------------------------------------------
-- Table Settings
-- -----------------------------------------------------
CREATE TABLE Settings (
  DbVersion INT NOT NULL,
  DateCreated TIMESTAMP NULL,
  DateModified TIMESTAMP NULL,
  ServerName VARCHAR(50) NULL,
  ServerDescription VARCHAR(50) NULL,
  ServerVersion VARCHAR(50) NULL,
  Environment VARCHAR(50) NULL,
  PortalBaseUrl VARCHAR(255) NULL,
  PhpMyAdminUrl VARCHAR(255) NULL,
  OwnerUrl VARCHAR(255) NULL,
  FooterText VARCHAR(255) NULL)

-- -----------------------------------------------------
-- Table WebLog
-- -----------------------------------------------------
CREATE TABLE WebLog (
  ID INT NOT NULL,
  UserIdentity VARCHAR(255) NULL,
  SeverityLevel INT NULL,
  DateCreated TIMESTAMP NULL,
  Message CLOB NULL,
  PRIMARY KEY (ID))

CREATE SEQUENCE WEBLOG_ID_SEQ;
  
 CREATE OR REPLACE TRIGGER WEBLOG_ID_TRIG 
	BEFORE INSERT ON WebLog 
	FOR EACH ROW
	WHEN (new.ID IS NULL)
	BEGIN
	  SELECT WEBLOG_ID_SEQ.NEXTVAL
	  INTO   :new.ID
	  FROM   dual;
	END;

-- -----------------------------------------------------
-- Table WebSeverityLevel
-- -----------------------------------------------------
CREATE TABLE WebSeverityLevel (
  Level INT NOT NULL,
  Description VARCHAR(20) NULL)

DELETE FROM WebSeverityLevel;
INSERT INTO WebSeverityLevel (Level,Description) VALUES (0,'Debug');
INSERT INTO WebSeverityLevel (Level,Description) VALUES (1,'Info');
INSERT INTO WebSeverityLevel (Level,Description) VALUES (2,'Warning');
INSERT INTO WebSeverityLevel (Level,Description) VALUES (3,'Error');

DELETE FROM Settings;
INSERT INTO Settings (DbVersion,DateCreated,DateModified,ServerName,ServerDescription,ServerVersion,Environment,PortalBaseUrl,PhpMyAdminUrl,OwnerUrl,FooterText) SELECT 2,CURRENT_TIMESTAMP,CURRENT_TIMESTAMP,'Node.js server','Application for manage quizzes.','4.1.0','DEV','http://localhost:3313/',NULL,'https://www.quizfaber.com','QuizFaber' FROM DUAL;

INSERT INTO PersonRole (ID,RoleName,RoleDescription,IsEnabled) SELECT * FROM ( SELECT 1 AS ID,'admin' AS RoleName,'administrator role' AS RoleDescription,1 AS IsEnabled FROM DUAL) WHERE NOT EXISTS (SELECT RoleName FROM PersonRole WHERE RoleName = 'admin');
INSERT INTO PersonRole (ID,RoleName,RoleDescription,IsEnabled) SELECT * FROM ( SELECT 2 AS ID,'user' AS RoleName,'user role' AS RoleDescription,1 AS IsEnabled FROM DUAL) WHERE NOT EXISTS (SELECT RoleName FROM PersonRole WHERE RoleName = 'user');

-- password : admin
INSERT INTO Person (PersonName,UserIdentity,UserPassword,UserRole,IsEnabled) SELECT * FROM (SELECT 'Administrator' AS PersonName,'admin' AS UserIdentity,'21232f297a57a5a743894a0e4a801fc3' AS UserPassword,1 AS UserRole,1 AS IsEnabled FROM DUAL) WHERE NOT EXISTS (SELECT UserIdentity FROM Person WHERE UserIdentity = 'admin');
-- password : anonymous
INSERT INTO Person (PersonName,UserIdentity,UserPassword,UserRole,IsEnabled) SELECT * FROM (SELECT 'Anonymous' AS PersonName, 'anonymous@anonymous.nowhere' AS UserIdentity,'294de3557d9d00b3d2d8a1e6aab028cf' AS UserPassword,2 AS UserRole,1 AS IsEnabled FROM DUAL) WHERE NOT EXISTS (SELECT UserIdentity FROM Person WHERE UserIdentity = 'anonymous@anonymous.nowhere');
